/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   http://www.tkiraaly.hu
*   Date         -  2014.11.01.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR)
*
*   Motor vezerles
*   ATmega8, LED 
*
*   Ez egy soros portrol vezrelhet motorvezerles.
*   A soros port 9600N1 bellts.
*   Parancsok:
*   A    - allj
*   B    - balra indul
*   J    - jobbra indul
*   
********************************************************************************
*   PonyProg Configuration and Security Bits (bepipalva):
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*
*   Calibrated Internal RC Oscillator
*   SUT1  SUT0
*   -     P         Slowly rising power
*   
********************************************************************************
*     
*    Soros port:
*     1 VCC
*     2 X
*     3 RX             - AVR RXD  - 2
*     4 TX             - ACR TXD  - 3
*     5 GND
*
*    Programozo fej:
*     1 RST            - AVR NRES - 1
*     2 X
*     3 SCK            - AVR SCK  - 19
*     4 MISO           - AVR MISO - 18
*     5 MOSI           - AVR MOSI - 17
*     6 GND
*
*    Egyeb:
*     LED              - AVR PB0 - 14
*     Csipogo          - AVR PC3 - 26
*     Motor irany      - AVR PB1 - 15
*     Motor be/ki      - AVR PB2 - 16
*
*******************************************************************************/

#define F_CPU                _4MHZ




#include "tkiraaly_atmega8.h"
#include <util/delay.h>




// LED - PB0
#define LED_ENABLE           PB0_OUT
#define LED_BE               BC( PORTB, 0)
#define LED_KI               BS( PORTB, 0)


// MOTOR
#define MOTOR_ENABLE         PB1_OUT;PB2_OUT
#define MOTOR_LEFT           BC( PORTB, 1)
#define MOTOR_RIGHT          BS( PORTB, 1)




void motor_start( void);               // motor inditasa
void motor_stop( void);                // motor leallitasa




ISR( USART_RXC_vect)                   // USART megszakitas kezelese
{
   switch( UDR)
   {
      case 'B':
      case 'b':
                motor_stop();
                _delay_ms( 100);
                MOTOR_LEFT;
                motor_start();
                break;
      case 'A':
      case 'a':
                motor_stop();
                break;
      case 'J':
      case 'j':
                motor_stop();
                _delay_ms( 100);
                MOTOR_RIGHT;
                motor_start();
                break;
   }
}    




int main( void)
{
   LED_ENABLE;
   LED_BE;
   MOTOR_LEFT;
   motor_stop();
   MOTOR_ENABLE;
   USART_SET_9600_8N1;
   IT_ENABLE;
   for(;;);
   return 0;
}




void motor_start( void)
{
   if( BTC( PORTB, 1)) BS(PORTB, 2);
   else BC( PORTB, 2); 
}




void motor_stop( void)
{
   if( BTC(PORTB, 1)) BC( PORTB, 2);
   else BS( PORTB, 2); 
}
