/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2014.08.17.
*   Chip         -  Atmel ATmega8
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   Definitions for ATMega8
*
********************************************************************************
*
*                              +---------+
*               NRESET - PC6  ++ 1    28 |  PC5 - ADC5 - SCL
*                  RXD - PD0  |  2    27 |  PC4 - ADC4 - SDA
*                  TXD - PD1  |  3    26 |  PC3 - ADC3
*                 INT0 - PD2  |  4    25 |  PC2 - ADC2
*                 INT1 - PD3  |  5    24 |  PC1 - ADC1
*             XCK - T0 - PD4  |  6    23 |  PC0 - ADC0
*                        VCC  |  7    22 |  GND
*                        GND  |  8    21 |  AREF
*         XTAL1 - OSC1 - PB6  |  9    20 |  AVCC
*         XTAL2 - OSC2 - PB7  | 10    19 |  PB5 - SCK
*                   T1 - PD5  | 11    18 |  PB4 - MISO
*                 AIN0 - PD6  | 12    17 |  PB3 - MOSI - OC2
*                 AIN1 - PD7  | 13    16 |  PB2 - NSS - OC1B
*                 ICP1 - PB0  | 14    15 |  PB1 - OC1A
*                             +----------+
*
********************************************************************************
*   PonyProg Configuration and Security Bits (P - bepipalva):
*
*   BootLock12        - Boot Lock Bit1
*   BootLock11
*   BootLock02        - Boot Lock Bit0
*   BootLock02
*   LOCK2             - Memory protection
*   LOCK1
*
*   RSTDISBLE         - Select if PC6 is I/O pin or RESET pin
*   WDTON             - Watchdog Timer always on
*   SPIEN             - Enable Serial Program and Data Downloading
*   CKOPT             - Oscillator options
*   EESAVE            - EEPROM memory is preserved through the Chip Erase
*   BOOTESZ1          - Select Boot Size
*   BOOTESZ0
*   BOOTRST           - Select Reset Vector
*
*   BODLEVEL          - 2.7V / 4V (P)
*   BODEN             - Brown-out detection disable / enable (P) 
*   SUT1              - Select start-up time
*   SUT0
*   CKSEL3            - Select Clock source
*   CKSEL2
*   CKSEL1
*   CKSEL0
*
*   PonyProg Configuration and Security Bits (bepipalva):
*
*   CKSEL3, CKSEL2, CKSEL1, CKSEL0
*   P       P       P       P       0000  External Clock
*   
*   P       P       P       -       0001  Calibrated Internal RC Oscillator 1MHz
*   P       P       -       P       0010  Calibrated Internal RC Oscillator 2MHz
*   P       P       -       -       0011  Calibrated Internal RC Oscillator 4MHz
*   P       -       P       P       0100  Calibrated Internal RC Oscillator 8MHz
*   P       -       P       -       0101  External RC Oscillator 0.1-0.9MHz
*   P       -       -       P       0110  External RC Oscillator 0.9-3MHz
*   P       -       -       -       0111  External RC Oscillator 3-8MHz
*   -       P       P       P       1000  External RC Oscillator 8MHz-
*   -       P       P       -       1001  External Low-frequency Crystal
*   -       P       -       P       1010  Ceramic resonator 0.4-0.9MHz
*   -       P       -       -       1011  Crystal Oscillator 0.4-0.9MHz
*   -       -       P       P       1100  Ceramic resonator 0.9-3MHz
*   -       -       P       -       1101  Crystal Oscillator 0.9-3MHz
*   -       -       -       P       1110  Ceramic resonator 3-8MHz
*   -       -       -       P       1110  Ceramic resonator 8MHz-, +CKOPT P
*   -       -       -       -       1111  Crystal Oscillator 3-8MHz
*   -       -       -       -       1111  Crystal Oscillator 8MHz-, +CKOPT P
*
*   Calibrated Internal RC Oscillator
*   SUT1   SUT0
*   P      P        00  BOD enabled
*   P      -        01  Fast rising power
*   -      P        10  Slowly rising power
*   -      -        11  Reserved
*
*   Ceramic resonator
*   SUT1   SUT0
*   P      P        00  Fast rising power
*   P      -        01  Slowly rising power
*   -      P        10  BOD enabled
*   -      -        11  Fast rising power

*   External Crystal
*   SUT1   SUT0
*   P      P        00  Slowly rising power
*   P      -        01  BOD enabled
*   -      P        10  Fast rising power
*   -      -        11  Slowly rising power
*
*   LOCK2   LOCK1
*   -       -       11  No memory lock
*   -       P       10  Programming disable
*   P       P       11  Programming and verification disable
*
*   Boot loader size
*   BOOTESZ1  BOOTESZ0
*   -         -         11   128 world
*   -         P         10   256 world
*   P         -         01   512 world
*   P         P         00  1024 world
*
*******************************************************************************/

#include <avr/sfr_defs.h>


#define TWBR    _SFR_IO8( 0x00)        // TWI stands for "Two Wire Interface" or "TWI Was I2C(tm)"

#define TWSR    _SFR_IO8( 0x01)
#define TWS7    7
#define TWS6    6
#define TWS5    5
#define TWS4    4
#define TWS3    3
// bit 2 reserved
#define TWPS1   1
#define TWPS0   0

#define TWAR    _SFR_IO8( 0x02)
#define TWA6    7
#define TWA5    6
#define TWA4    5
#define TWA3    4
#define TWA2    3
#define TWA1    2
#define TWA0    1
#define TWGCE	  0

#define TWDR    _SFR_IO8( 0x03)

#define ADCW    _SFR_IO16( 0x04)       // ADC Data Register
#define ADCL    _SFR_IO8( 0x04)        // ADC Data Register L
#define ADCH    _SFR_IO8( 0x05)        // ADC Data Register H

#define ADCSR   _SFR_IO8( 0x06)
#define ADCSRA  _SFR_IO8( 0x06)        // ADC Control and Status Register A
#define ADEN    7
#define ADSC    6
#define ADFR    5
#define ADIF    4
#define ADIE    3
#define ADPS2   2
#define ADPS1   1
#define ADPS0   0

#define ADMUX   _SFR_IO8(0x07)         // ADC Multiplexer Selection Register
#define REFS1   7
#define REFS0   6
#define ADLAR   5
// bit 4 reserved
#define MUX3    3
#define MUX2    2
#define MUX1    1
#define MUX0    0

#define ACSR    _SFR_IO8( 0x08)        // analog comparator
#define ACD     7
#define ACBG    6
#define ACO     5
#define ACI     4
#define ACIE    3
#define ACIC    2
#define ACIS1   1
#define ACIS0   0

#define UBRRL   _SFR_IO8( 0x09)        // USART Baud Rate Register Low

#define UCSRB   _SFR_IO8( 0x0A)        // USART Control and Status Register B
#define RXCIE   7                      // RX Complete Interrupt Enable
#define TXCIE   6                      // TX Complete Interrupt Enable
#define UDRIE   5                      // USART Data Register Empty Interrupt Enable
#define RXEN    4                      // Receive Enable
#define TXEN    3                      // Transmitter Enable
#define UCSZ2   2                      // Character Size
#define RXB8    1                      // Receive Data Bit 8
#define TXB8    0                      // Transmit Data Bit 8

#define UCSRA   _SFR_IO8( 0x0B)        // USART Control and Status Register A
#define RXC     7                      // USART Receive Complete
#define TXC     6                      // USART Transmit Complete
#define UDRE    5                      // USART Data Register Empty
#define FE      4                      // Frame Error 
#define DOR     3                      // Data Overrun
#define PE      2                      // USART Parity Error
#define U2X     1                      // Double the USART Transmission Speed
#define MPCM    0                      // Multiprocessor Communication Mode

#define UDR     _SFR_IO8( 0x0C)        // USART I/O Data Register

#define SPCR    _SFR_IO8( 0x0D)        // SPI
#define SPIE    7
#define SPE     6
#define DORD    5
#define MSTR    4
#define CPOL    3
#define CPHA    2
#define SPR1    1
#define SPR0    0

#define SPSR    _SFR_IO8( 0x0E)
#define SPIF    7
#define WCOL    6
#define SPI2X   0

#define SPDR    _SFR_IO8( 0x0F)

#define PIND    _SFR_IO8( 0x10)        // Port D - Input Register

#define DDRD    _SFR_IO8( 0x11)        // Port D - Data Direction Register 

#define PORTD   _SFR_IO8( 0x12)        // Port D - Output Register

#define PINC    _SFR_IO8( 0x13)        // Port C - Input Register

#define DDRC    _SFR_IO8( 0x14)        // Port C - Data Direction Register

#define PORTC   _SFR_IO8( 0x15)        // Port C - Output Register

#define PINB    _SFR_IO8( 0x16)        // Port B - Input Register

#define DDRB    _SFR_IO8( 0x17)        // Port B - Data Direction Register

#define PORTB   _SFR_IO8( 0x18)        // Port B - Output Register

#define EECR    _SFR_IO8( 0x1C)        // EEPROM Control Register
#define EERIE   3                      // EEPROM Ready Interrupt Enabla Bit
#define EEMPE   2                      // EEPROM Master Program Enable Bit
#define EEPE    1                      // EEPROM Program Enable Bit
#define EERE    0                      // EEPROM Read Enable Bit

#define EEDR    _SFR_IO8( 0x1D)        // EEPROM Data Register

#define EEAR    _SFR_IO16( 0x1E)       // EEPROM Address Register
#define EEARL   _SFR_IO8( 0x1E)
#define EEARH   _SFR_IO8( 0x1F)

#define UCSRC   _SFR_IO8( 0x20)
#define URSEL   7
#define UMSEL   6
#define UPM1    5
#define UPM0    4
#define USBS    3
#define UCSZ1   2
#define UCSZ0   1
#define UCPOL   0

#define UBRRH   _SFR_IO8( 0x20)
#define URSEL   7

#define WDTCR   _SFR_IO8( 0x21)        // Watchdog Timer Control Register
// bits 7-5 reserved
#define WDCE    4
#define WDE     3
#define WDP2    2
#define WDP1    1
#define WDP0    0

#define ASSR    _SFR_IO8( 0x22)
// bits 7-4 reserved
#define AS2	3
#define TCN2UB	2
#define OCR2UB	1
#define TCR2UB	0

#define OCR2    _SFR_IO8( 0x23)        // Timer 2

#define TCNT2   _SFR_IO8( 0x24)

#define TCCR2   _SFR_IO8( 0x25)        // Timer/Counter 2 Control Register
#define FOC2	  7
#define WGM20	  6
#define COM21	  5
#define COM20	  4
#define WGM21	  3
#define CS22	  2
#define CS21	  1
#define CS20	  0

#define ICR1	 _SFR_IO16( 0x26)        // Input Capture Register 1
#define ICR1L	 _SFR_IO8( 0x26)
#define ICR1H  _SFR_IO8( 0x27)

#define OCR1B  _SFR_IO16( 0x28)        // Output Compare Register 1 B
#define OCR1BL _SFR_IO8( 0x28)
#define OCR1BH _SFR_IO8( 0x29)

#define OCR1A	 _SFR_IO16( 0x2A)        // Output Compare Register 1 A
#define OCR1AL _SFR_IO8( 0x2A)
#define OCR1AH _SFR_IO8( 0x2B)

#define TCNT1  _SFR_IO16( 0x2C)        // Timer/Counter 1
#define TCNT1L _SFR_IO8( 0x2C)
#define TCNT1H _SFR_IO8( 0x2D)

#define TCCR1B  _SFR_IO8( 0x2E)        // Timer/Counter 1 Control Register B
#define ICNC1   7
#define ICES1   6
// bit 5 reserved
#define WGM13   4
#define WGM12   3
#define CS12    2
#define CS11    1
#define CS10    0

#define TCCR1A  _SFR_IO8( 0x2F)       // Timer/Counter 1 Control Register A
#define COM1A1  7
#define COM1A0  6
#define COM1B1  5
#define COM1B0  4
#define FOC1A   3
#define FOC1B   2
#define WGM11   1
#define WGM10   0

#define SFIOR   _SFR_IO8( 0x30)        // Special Function IO Register
#define ACME    3
#define PUD     2                      // Pull-up Disable
#define PSR2    1
#define PSR10   0

#define OSCCAL  _SFR_IO8( 0x31)        // Oscillator Calibration Register

#define TCNT0   _SFR_IO8( 0x32)        // Timer/Counter Register 0 

#define TCCR0   _SFR_IO8( 0x33)        // Timer/Counter Control Register
#define CS02    2                      // Clock Set 2
#define CS01    1                      // Clock Set 1
#define CS00    0                      // Clock Set 0

#define MCUCSR  _SFR_IO8( 0x34)        // MCU Status Register
#define WDRF    3
#define BORF    2
#define EXTRF   1
#define PORF    0

#define MCUCR   _SFR_IO8( 0x35)        // MCU Control Register
#define SE      7
#define SM2     6
#define SM1     5
#define SM0     4
#define ISC11   3
#define ISC10   2
#define ISC01   1
#define ISC00   0

#define TWCR    _SFR_IO8( 0x36)
#define TWINT   7
#define TWEA    6
#define TWSTA   5
#define TWSTO   4
#define TWWC    3
#define TWEN    2
// bit 1 reserved (TWI_TST?)
#define TWIE    0

#define SPMCR   _SFR_IO8( 0x37)        // Store Program Memory Control Register
#define SPMIE   7
#define RWWSB   6
// bit 5 reserved
#define RWWSRE  4
#define BLBSET  3
#define PGWRT   2
#define PGERS   1
#define SPMEN   0

#define TIFR    _SFR_IO8( 0x38)        // Timer/Counter Interrupt Flag Register
#define OCF2    7
#define TOV2    6
#define ICF1    5
#define OCF1A   4
#define OCF1B   3
#define TOV1    2
// bit 1 reserved (OCF0?)
#define TOV0    0

#define TIMSK   _SFR_IO8( 0x39)        // Timer/Counter Interrupt Mask Register
#define OCIE2   7
#define TOIE2	  6
#define TICIE1	5
#define OCIE1A	4
#define OCIE1B	3
#define TOIE1   2
// bit 1 reserved (OCIE0?)
#define TOIE0   0

#define GIFR    _SFR_IO8( 0x3A)
#define INTF1   7
#define INTF0	  6

#define GICR	  _SFR_IO8( 0x3B)         // General Interrupt Control Register
#define INT1    7                       // External Interrupt Request 1 Enable
#define INT0    6                       // External Interrupt Request 0 Enable
#define IVSEL   1                       // External Interrupt Flag 1
#define IVCE    0                       // External Interrupt Flag 0


// 0x3C reserved (OCR0?)
// 0x3D..0x3E SP
// 0x3F SREG


// Interrupt vectors:
#define INT0_vect         _VECTOR( 1)  // External Interrupt Request 0
#define IT_INT0_ENABLE             BS(GICR,INT0)
#define IT_INT0_DISABLE            BC(GICR,INT0)

#define INT1_vect         _VECTOR( 2)  // External Interrupt Request 1
#define IT_INT1_ENABLE           BS(GICR,INT1)
#define IT_INT1_DISABLE          BC(GICR,INT1)

#define TIMER2_COMP_vect  _VECTOR( 3)  // Timer/Counter2 Compare Match
#define IT_TIMER2_COMP_ENABLE    BS(TIMSK,OCIE2)
#define IT_TIMER2_COMP_DISABLE   BC(TIMSK,OCIE2)

#define TIMER2_OVF_vect   _VECTOR( 4)  // Timer/Counter2 Overflow
#define IT_TIMER2_OVF_ENABLE     BS(TIMSK,TOIE2)
#define IT_TIMER2_OVF_DISABLE    BC(TIMSK,TOIE2)

#define TIMER1_CAPT_vect	_VECTOR( 5)  // Timer/Counter1 Capture Event
#define IT_TIMER1_CAPT_ENABLE    BS(TIMSK,TICIE1)
#define IT_TIMER1_CAPT_DISABLE   BC(TIMSK,TICIE1)

#define TIMER1_COMPA_vect	_VECTOR( 6)  // Timer/Counter1 Compare Match A
#define IT_TIMER1_COMPA_ENABLE   BS(TIMSK,OCIE1A)
#define IT_TIMER1_COMPA_DISABLE  BC(TIMSK,OCIE1A)

#define TIMER1_COMPB_vect	_VECTOR( 7)  // Timer/Counter1 Compare Match B
#define IT_TIMER1_COMPB_ENABLE   BS(TIMSK,OCIE1B)
#define IT_TIMER1_COMPB_DISABLE  BC(TIMSK,OCIE1B)

#define TIMER1_OVF_vect		_VECTOR( 8)  // Timer/Counter1 Overflow
#define IT_TIMER1_OVF_ENABLE     BS(TIMSK,TOIE1)
#define IT_TIMER1_OVF_DISABLE    BC(TIMSK,TOIE1)

#define TIMER0_OVF_vect		_VECTOR( 9)  // Timer/Counter0 Overflow
#define IT_TIMER0_OVF_ENABLE     BS(TIMSK,TOIE0)
#define IT_TIMER0_OVF_DISABLE    BC(TIMSK,TOIE0)

#define SPI_STC_vect			_VECTOR( 10) // Serial Transfer Complete
#define IT_SPI_STC_ENABLE        BS(SPCR,SPIE)
#define IT_SPI_STC_DISABLE       BC(SPCR,SPIE)

#define USART_RXC_vect		_VECTOR( 11) // USART, Rx Complete
#define IT_USART_RXC_ENABLE      BS(UCSRB,RXCIE)
#define IT_USART_RXC_DISABLE     BC(UCSRB,RXCIE)

#define USART_UDRE_vect		_VECTOR( 12) // USART Data Register Empty
#define IT_USART_UDRE_ENABLE     BS(UCSRB,UDRIE)
#define IT_USART_UDRE_DISABLE    BC(UCSRB,UDRIE)

#define USART_TXC_vect		_VECTOR( 13) // USART, Tx Complete
#define IT_USART_TXC_ENABLE      BS(UCSRB,TXCIE)
#define IT_USART_TXC_DISABLE     BC(UCSRB,TXCIE)

#define ADC_vect			    _VECTOR( 14) // ADC Conversion Complete
#define IT_ADC_ENABLE            BS(ADCSRA,ADIE)
#define IT_ADC_DISABLE           BC(ADCSRA,ADIE)

#define EE_RDY_vect			  _VECTOR( 15) // EEPROM Ready
#define IT_EE_RDY_ENABLE         BS(EECR,EERIE)
#define IT_EE_RDY_DISABLE        BC(EECR,EERIE)

#define ANA_COMP_vect			_VECTOR( 16) // Analog Comparator
#define IT_ANA_COMP_ENABLE       BS(ACSR,ACIE)
#define IT_ANA_COMP_DISABLE      BC(ACSR,ACIE)

#define TWI_vect			    _VECTOR( 17) // 2-wire Serial Interface
#define IT_TWI_ENABLE            BS(TWCR,TWIE)
#define IT_TWI_DISABLE           BC(TWCR,TWIE)

#define SPM_RDY_vect			_VECTOR( 18) // Store Program Memory Ready
#define IT_SPM_RDY_ENABLE        BS(SPMCR,SPMIE)
#define IT_SPM_RDY_DISABLE       BC(SPMCR,SPMIE)


#define ISR(vector) void vector (void) __attribute__ ((signal,used)); void vector (void)


/* Constants */
#define _VECTORS_SIZE 38
#define SPM_PAGESIZE    64
#define RAMEND       0x45F
#define XRAMEND     RAMEND
#define E2END        0x1FF
#define E2PAGESIZE       4
#define FLASHEND    0x1FFF


/* Fuses */
#define FUSE_MEMORY_SIZE 2

/* Low Fuse Byte */
#define FUSE_CKSEL0      (unsigned char)~_BV(0)
#define FUSE_CKSEL1      (unsigned char)~_BV(1)
#define FUSE_CKSEL2      (unsigned char)~_BV(2)
#define FUSE_CKSEL3      (unsigned char)~_BV(3)
#define FUSE_SUT0        (unsigned char)~_BV(4)
#define FUSE_SUT1        (unsigned char)~_BV(5)
#define FUSE_BODEN       (unsigned char)~_BV(6)
#define FUSE_BODLEVEL    (unsigned char)~_BV(7)
#define LFUSE_DEFAULT (FUSE_SUT0 & FUSE_CKSEL3 & FUSE_CKSEL2 & FUSE_CKSEL1)

/* High Fuse Byte */
#define FUSE_BOOTRST     (unsigned char)~_BV(0)
#define FUSE_BOOTSZ0     (unsigned char)~_BV(1)
#define FUSE_BOOTSZ1     (unsigned char)~_BV(2)
#define FUSE_EESAVE      (unsigned char)~_BV(3)
#define FUSE_CKOPT       (unsigned char)~_BV(4)
#define FUSE_SPIEN       (unsigned char)~_BV(5)
#define FUSE_WDTON       (unsigned char)~_BV(6)
#define FUSE_RSTDISBL    (unsigned char)~_BV(7)
#define HFUSE_DEFAULT (FUSE_SPIEN & FUSE_BOOTSZ1 & FUSE_BOOTSZ0)


/* Lock Bits */
#define __LOCK_BITS_EXIST
#define __BOOT_LOCK_BITS_0_EXIST
#define __BOOT_LOCK_BITS_1_EXIST 


/* Signature */
#define SIGNATURE_0 0x1E
#define SIGNATURE_1 0x93
#define SIGNATURE_2 0x07




// KT macro-k

#define U8                   unsigned char
#define U16                  unsigned int

#define KHZ                  *1000
#define MHZ                  *1000000
#define _4MHZ                4000000
#define _8MHZ                8000000
#define _16MHZ               16000000

#define BS(sfr,bit)          sfr|= ( 1 << ( bit))               // BIT-SET - egy regiszter adott bitjenek 1-re allitas
#define BC(sfr,bit)          sfr&= ~( 1 << ( bit))              // BIT-CLEAR - egy regiszter adott bitjenek 0-ra allitas
#define BTS(sfr,bit)         bit_is_set( sfr, bit)              // BIT-TEST-SET - egy regiszter egy bitjenek vizsgalata 1-re
#define BTC(sfr,bit)         bit_is_clear( sfr, bit)            // BIT-TEST-CLEAR - egy regiszter egy bitjenek vizsgalata 0-ra

#define IT_ENABLE            __asm__ __volatile__ ("sei" ::)    // megszakitas engedelyezese
#define IT_DISABLE           __asm__ __volatile__ ("cli" ::)    // megszakitas tiltasa
#define NOP                  __asm__ __volatile__ ("nop" ::)    // egy ures utasitas beszurasa




// Port labak kimenetre allitasa

#define PB0_OUT              BS( DDRB, 0)          
#define PB1_OUT              BS( DDRB, 1)
#define PB2_OUT              BS( DDRB, 2)
#define PB3_OUT              BS( DDRB, 3)
#define PB4_OUT              BS( DDRB, 4)
#define PB5_OUT              BS( DDRB, 5)
#define PB6_OUT              BS( DDRB, 6)
#define PB7_OUT              BS( DDRB, 7)

#define PC0_OUT              BS( DDRC, 0)
#define PC1_OUT              BS( DDRC, 1)
#define PC2_OUT              BS( DDRC, 2)
#define PC3_OUT              BS( DDRC, 3)
#define PC4_OUT              BS( DDRC, 4)
#define PC5_OUT              BS( DDRC, 5)
#define PC6_OUT              BS( DDRC, 6)

#define PD0_OUT              BS( DDRD, 0)
#define PD1_OUT              BS( DDRD, 1)
#define PD2_OUT              BS( DDRD, 2)
#define PD3_OUT              BS( DDRD, 3)
#define PD4_OUT              BS( DDRD, 4)
#define PD5_OUT              BS( DDRD, 5)
#define PD6_OUT              BS( DDRD, 6)
#define PD7_OUT              BS( DDRD, 7)




// USART-hoz

// UCSRA beallitasai
#define DOUBLE_SPEED         0B00000010          // ketszeres sebesseg

// UCSRB beallitasai
#define IT_RX_ENABLE         0B10000000          // vetel kesz megszakitas engedelyezese
#define IT_TX_ENABLE         0B01000000          // adas kesz megszakitas engedelyezese
#define IT_UDR_EMPTY_ENABLE  0B00100000          // adat buffer ures megszakitas engedelyezese
#define RX_ENABLE            0B00010000          // vetel engedelyezese
#define TX_ENABLE            0B00001000          // adas engedelyezese

// UCSRC beallitasai
#define SET_UCSRC            0B10000000          // UCSRC kivalasztasa
#define SET_UBRRH            0                   // UBRRB kivalasztasa
#define ASYNCRON             0                   // asszinkron mukodes
#define SYNCRON              0B01000000          // szinkron mukodes
#define PARITY_NO            0                   // nincs paritas bit
#define PARITY_EVEN          0B00100000          // paros paritas
#define PARITY_ODD           0B00110000          // paratlan paritas
#define STOPBIT_1            0                   // 1 stopbit
#define STOPBIT_2            0B00001000          // 2 stopbit
#define BIT_5                0                   // 5 bit-es karakterhossz
#define BIT_6                0B00000010          // 6 bit-es karakterhossz
#define BIT_7                0B00000100          // 7 bit-es karakterhossz
#define BIT_8                0B00000110          // 8 bit-es karakterhossz

#define KTUBRR               F_CPU/ 16UL/ BAUDRATE - 1

#define USART_SET_9600_8N1   UBRRL= F_CPU/ 16UL/ 9600- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 9600- 1)>> 8;  \
                             UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE

#define USART_SET_19200_8N1  UBRRL= F_CPU/ 16UL/ 19200- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 19200- 1)>> 8;  \
                             UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE

#define USART_SET_115200_8N1 UBRRL= F_CPU/ 16UL/ 115200- 1;  \
                             UBRRH= ( F_CPU/ 16UL/ 115200- 1)>> 8;  \
                             UCSRC= SET_UCSRC+ ASYNCRON+ BIT_8+ PARITY_NO+ STOPBIT_1;  \
                             UCSRB= RX_ENABLE+ TX_ENABLE+ IT_RX_ENABLE




// MCUCR beallitasai
#define INT0_LEVEL_0         MCUCR&= 0B11111100
#define INT0_ALL_EDGES       MCUCR&= 0B11111100; MCUCR|= 0B00000001
#define INT0_FALLING         MCUCR&= 0B11111100; MCUCR|= 0B00000010
#define INT0_RISING                              MCUCR|= 0B00000011

#define INT1_LEVEL_0         MCUCR&= 0B11110011
#define INT1_ALL_EDGES       MCUCR&= 0B11110011; MCUCR|= 0B00000100
#define INT1_FALLING         MCUCR&= 0B11110011; MCUCR|= 0B00001000
#define INT1_RISING                              MCUCR|= 0B00001100




// ADC-hez

// ADC orajele 50-200kHz, ha nincs definialva, ugy valasztja hogy lehetoleg nagy legyen
#ifndef ADC_DIVIDER                        // 2-4-8-16-32-64-128
   #if F_CPU > 12 MHZ
      #define ADC_DIVIDER                  128
   #else
      #if F_CPU > 6 MHZ
         #define ADC_DIVIDER                64
      #else
         #if F_CPU > 3 MHZ
            #define ADC_DIVIDER             32
         #else
            #if F_CPU > 1,5 MHZ
               #define ADC_DIVIDER          16 
            #else
               #if F_CPU > 0,75 MHZ
                  #define ADC_DIVIDER        8
               #else
                  #if F_CPU > 0,37 MHZ
                     #define ADC_DIVIDER     4
                  #else
                     #define ADC_DIVIDER     2
                  #endif
               #endif
            #endif
         #endif
      #endif
   #endif
#endif

#if F_CPU/ADC_DIVIDER < 50000
   #warning "F_CPU / ADC_DIVIDER < 50000!"
#endif   
#if F_CPU/ADC_DIVIDER > 200000
   #warning "F_CPU / ADC_DIVIDER > 200000!"
#endif   

#define ADC_8BIT             BS( ADMUX, 5)                             // 8 bit felbontas, ADCH
#define ADC_10BIT            BC( ADMUX, 5)                             // 10 bit felbontas, ADCH+ ADCL

#define ADC_VREF_EXT         ADMUX&= 0B00111111                        // kulso referencia feszultseg ( AREF lab)
#define ADC_VREF_AVCC        ADMUX&= 0B00111111; ADMUX|= 0B01000000    // analog tap ( AVCC lab)
#define ADC_VREF_INT                             ADMUX|= 0B11000000    // belso 2V56

#define ADC_IN0              ADMUX&= 0B11110000
#define ADC_IN1              ADMUX&= 0B11110000; ADMUX|= 0B00000001
#define ADC_IN2              ADMUX&= 0B11110000; ADMUX|= 0B00000010
#define ADC_IN3              ADMUX&= 0B11110000; ADMUX|= 0B00000011
#define ADC_IN4              ADMUX&= 0B11110000; ADMUX|= 0B00000100
#define ADC_IN5              ADMUX&= 0B11110000; ADMUX|= 0B00000101
#define ADC_IN6              ADMUX&= 0B11110000; ADMUX|= 0B00000110
#define ADC_IN7              ADMUX&= 0B11110000; ADMUX|= 0B00000111
#define ADC_IN_1V3           ADMUX&= 0B11110000; ADMUX|= 0B00001110
#define ADC_IN_0V                                ADMUX|= 0B00001111

#define ADC_ENABLE           BS( ADCSRA, 7)
#define ADC_DISABLE          BC( ADCSRA, 7)
#define ADC_FINISHED         BTS( ADCSRA, 4)
#define ADC_IT_ENABLE        BS( ADCSRA, 3)
#define ADC_IT_DISABLE       BC( ADCSRA, 3)

#if ADC_DIVIDER == 2
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000001   // ADC indit
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100001   // ADC folyamatos
#endif
#if ADC_DIVIDER == 4
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000010
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100010
#endif
#if ADC_DIVIDER == 8
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000011
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100011
#endif
#if ADC_DIVIDER == 16
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000100
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100100
#endif
#if ADC_DIVIDER == 32
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000101
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100101
#endif
#if ADC_DIVIDER == 64
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000110
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100110
#endif
#if ADC_DIVIDER == 128
   #define ADC_START         ADCSRA&= 0B10111000; ADCSRA|= 0B01000111
   #define ADC_FREE_RUN      ADCSRA&= 0B11101000; ADCSRA|= 0B00100111
#endif




// T0 8 bit-es szamlalo, a legegyszerubb. 
// Elsosorban kulso impulzusok szamlalasara es idozitesre alkalmas.
// OVF IT-t akkor general, amikor TCNT0 255-rol 0-ra ugrik.
#define T0_RESET             TCNT0= 0
#define T0_STOP              BC(TCCR0,CS02); BC(TCCR0,CS01); BC(TCCR0,CS00)
#define T0_FP_1              BC(TCCR0,CS02); BC(TCCR0,CS01); BS(TCCR0,CS00)     // rendszer orajel
#define T0_FP_8              BC(TCCR0,CS02); BS(TCCR0,CS01); BC(TCCR0,CS00)     // rendszer orajel/ 8
#define T0_FP_64             BC(TCCR0,CS02); BS(TCCR0,CS01); BS(TCCR0,CS00)     // rendszer orajel/ 64
#define T0_FP_256            BS(TCCR0,CS02); BC(TCCR0,CS01); BC(TCCR0,CS00)     // rendszer orajel/ 256
#define T0_FP_1024           BS(TCCR0,CS02); BC(TCCR0,CS01); BS(TCCR0,CS00)     // rendszer orajel/ 1024
#define T0_EXT_FALLING       BS(TCCR0,CS02); BS(TCCR0,CS01); BC(TCCR0,CS00)     // kulso orajel (T0) lefuto elere
#define T0_EXT_RISING        BS(TCCR0,CS02); BS(TCCR0,CS01); BS(TCCR0,CS00)     // kulso orajel (T0) felfuto elere

 

 
// T1 16 bit-es szamlalo, szamos uzemoddal (nem mind lett definialva).
// Regiszterek - TCNT1, TCCR1A, TCCR1B, OCR1A, OCR1B, ICR1, TIMSK, ACSR, TIFR, SFIOR  
//#define T1_CAPTURE_RISE         BS(TCCR1B,ICES1)
//#define T1_CAPTURE_FALL         BC(TCCR1B,ICES1)
//#define T1_CAPTURE_ACOMP        BS(ACSR,ACIC)
//#define T1_CAPTURE_ICP0         BC(ACSR,ACIC)
#define T0_T1_DIVIDER_RESET  BS(SFIOR,PSR10)                                    // kozos T0-lal
#define T1_NORMAL            BC(TCCR1B,WGM13); BC(TCCR1B,WGM12); BC(TCCR1A,WGM11); BC(TCCR1A,WGM10)  // Szamlalas
#define T1_CTC               BC(TCCR1B,WGM13); BS(TCCR1B,WGM12); BC(TCCR1A,WGM11); BC(TCCR1A,WGM10)  // OCRA0-ig szamol, majd IT
#define T1_PWM               BC(TCCR1B,WGM13); BS(TCCR1B,WGM12); BC(TCCR1A,WGM11); BS(TCCR1A,WGM10)  // Fast PWM, 0..FF kozott szamol
#define T1_PWM2              BS(TCCR1B,WGM13); BS(TCCR1B,WGM12); BS(TCCR1A,WGM11); BS(TCCR1A,WGM10)  // Fast PWM, 0..OCR0A kozott szamol
#define T1_PWM10             BC(TCCR1B,WGM13); BS(TCCR1B,WGM12); BS(TCCR1A,WGM11); BS(TCCR1A,WGM10)  // Fast PWM, 0..1FF kozott szamol
#define OC1A_ENABLE          PB1_OUT; BS(TCCR1A,COM1A1); BS(TCCR1A,COM1A0)      // PWM, OCR1A-hoz
#define OC1B_ENABLE          PB2_OUT; BS(TCCR1A,COM1B1); BS(TCCR1A,COM1B0)      // PWM, OCR1B-hez
#define T1_RESET             TCNT0= 0
#define T1_STOP              BC(TCCR1B,CS12); BC(TCCR1B,CS11); BC(TCCR1B,CS10)
#define T1_FP_1              BC(TCCR1B,CS12); BC(TCCR1B,CS11); BS(TCCR1B,CS10)  // rendszer orajel
#define T1_FP_8              BC(TCCR1B,CS12); BS(TCCR1B,CS11); BC(TCCR1B,CS10)  // rendszer orajel/ 8
#define T1_FP_64             BC(TCCR1B,CS12); BS(TCCR1B,CS11); BS(TCCR1B,CS10)  // rendszer orajel/ 64
#define T1_FP_256            BS(TCCR1B,CS12); BC(TCCR1B,CS11); BC(TCCR1B,CS10)  // rendszer orajel/ 256
#define T1_FP_1024           BS(TCCR1B,CS12); BC(TCCR1B,CS11); BS(TCCR1B,CS10)  // rendszer orajel/ 1024
#define T1_EXT_FALLING       BS(TCCR1B,CS12); BS(TCCR1B,CS11); BC(TCCR1B,CS10)  // kulso orajel (T1) lefuto elere
#define T1_EXT_RISING        BS(TCCR1B,CS12); BS(TCCR1B,CS11); BS(TCCR1B,CS10)  // kulso orajel (T1) felfuto elere
#define T1_CTC_F(hz,osztas)     OCR1=F_CPU/osztas/hz-1  
#define T1_CTC_T(usec,osztas)   OCR1=usec/((osztas*1000000)/F_CPU-1)  




// T2 8 bit-es szamlalo, tobbfele uzemoddal, de csak belso orajelrol jar.
// OVF IT-t akkor general, amikor TCNT0 255-rol 0-ra ugrik.
// COMP IT-t akkor general, amikor TCNT0 erteke OCR2 ertekevel megegyezik.
// NORMAL modban TCNT2 0..255-0 szamlal, erteke kozben atirhato.
// CTC modban TCNT2 0..OCR2-0 szamlal. 
// PWM modban TCNT2 0..255-0 szamlal, OCR2-ig OC2=0 (kimenet).  
// Regiszterek - TCNT2, TCCR2, OCR2, TIMSK, SFIOR, TIFR  
#define T2_NORMAL            BC(TCCR2,WGM21); BC(TCCR2,WGM20)                   // Szamlalas
#define T2_CTC               BC(TCCR2,WGM21); BS(TCCR2,WGM20)                   // OCR2-ig szamol, majd IT
#define T2_PWM               BS(TCCR2,WGM21); BS(TCCR2,WGM20)                   // Fast PWM, 0..FF kozott szamol
#define OC2_ENABLE           PC2_OUT; BS(TCCR2,COM21); BS(TCCR2,COM20)          // PWM, OC2-hoz
#define T2_DIVIDER_RESET     BS(SFIOR,PSR2)
#define T2_RESET             TCNT2= 0
#define T2_STOP              BC(TCCR2,CS22); BC(TCCR2,CS21); BC(TCCR2,CS20)
#define T2_FP_1              BC(TCCR2,CS22); BC(TCCR2,CS21); BS(TCCR2,CS20)     // rendszer orajel
#define T2_FP_8              BC(TCCR2,CS22); BS(TCCR2,CS21); BC(TCCR2,CS20)     // rendszer orajel/ 8
#define T2_FP_32             BC(TCCR2,CS22); BS(TCCR2,CS21); BS(TCCR2,CS20)     // rendszer orajel/ 64
#define T2_FP_64             BS(TCCR2,CS22); BC(TCCR2,CS21); BC(TCCR2,CS20)     // rendszer orajel/ 256
#define T2_FP_128            BS(TCCR2,CS22); BC(TCCR2,CS21); BS(TCCR2,CS20)     // rendszer orajel/ 1024
#define T2_FP_256            BS(TCCR2,CS22); BS(TCCR2,CS21); BC(TCCR2,CS20)     // kulso orajel (T0) lefuto elere
#define T2_FP_1024           BS(TCCR2,CS22); BS(TCCR2,CS21); BS(TCCR2,CS20)     // kulso orajel (T0) felfuto elere
#define T2_CTC_F(hz,osztas)    OCR2=F_CPU/osztas/hz-1  
#define T2_CTC_T(usec,osztas)  OCR2=usec/((osztas*1000000)/F_CPU-1)  
